@@ -1,8 +1,5 @@
 #' @title Automatically create a loon widget
-#' @description \code{l_ggplot()} wraps function \code{ggplot} with assigning a new class "lggplot" to the output
-#' \code{ggplot} object and returns a \code{lggplot} object. When a \code{ggplot} object is processed,
-#' S3 method \code{print.ggplot} is rendered, however, if a \code{lggplot} object is processed,
-#' S3 method \code{print.lggplot} will be rendered which will return a \code{loon} widget
+#' @description Create a \code{loon} widget with \code{ggplot} syntax
 #'
 #' @param data Default dataset to use for plot. If not already a data.frame, will be converted to one by fortify().
 #' If not specified, must be supplied in each layer added to the plot.
@@ -10,11 +7,18 @@
 #' If not specified, must be supplied in each layer added to the plot.
 #' @param ... Other arguments passed on to methods. Not currently used.
 #' @param environment DEPRECATED. Used prior to tidy evaluation.
-#' @seealso \code{\link{ggplot}}, \code{\link{ggplot2loon}}, \code{\link{print.lggplot}}
+#' @seealso \code{\link{ggplot}}, \code{\link{ggplot2loon}}, \code{\link{print.l_ggplot}}
 #' @export
-#' @return It will return a \code{lggplot} object with class \code{c("lggplot", "gg", "ggplot")}.
+#' @return It will return an \code{l_ggplot} object with class \code{c("l_ggplot", "gg", "ggplot")}.
 #' Then print a \code{loon} plot automatically.
 #'
+#' @details function \code{l_ggplot()} wraps function \code{ggplot()} with assigning an additional class "l_ggplot"
+#' to the output. The returned object is called an \code{l_ggplot} object.
+#' To draw a \code{ggplot} object, S3 method \code{print.ggplot} will be rendered
+#' so that a static graphic is displayed. While, for an \code{l_ggplot()} object,
+#' S3 method \code{print.l_ggplot} will be rendered
+#' which will return an interactive \code{loon} widget.
+#'
 #' @seealso \code{\link{loon.ggplot}}
 #' @examples
 #' if(interactive()) {
@@ -23,8 +27,8 @@
 #'       size = 4,
 #'       mapping = aes(color = factor(cyl))
 #'     )
-#' # p is a `lggplot` object, `print.lggplot(p)` will be called automatically.
-#' # Then, a `lggplot` object will be transformed to a `loon` widget
+#' # p is an `l_ggplot` object, `print.l_ggplot(p)` will be called automatically.
+#' # Then, at printing time, an `l_ggplot` object will be transformed to a `loon` widget
 #' p
 #'
 #' \dontrun{
@@ -35,7 +39,7 @@
 #' q
 #' }
 #'
-#' # An alternative way to return a real loon widget from `p` (a `lggplot` object)
+#' # An alternative way to return a real loon widget from `p` (a `l_ggplot` object)
 #' # is to call the function `loon.ggplot`. Compared with calling function `l_getFrompath`
 #' # this way can provide richer information (note that it will create a new widget).
 #' q <- loon.ggplot(p)
@@ -64,7 +68,7 @@ l_ggplot <- function(data = NULL, mapping = aes(), ...,
               environment = environment)
 
   structure(p,
-            class = c("lggplot", "gg", "ggplot"))
+            class = c("l_ggplot", "gg", "ggplot"))
 }
 
 #' Explicitly draw plot
@@ -73,10 +77,14 @@ l_ggplot <- function(data = NULL, mapping = aes(), ...,
 #' @param ... other arguments used to modify function \code{ggplot2loon}
 #' @return Invisibly returns a \code{loon} widget
 #' @export
-print.lggplot <- function(x, ...) {
+print.l_ggplot <- function(x, ...) {
 
   p <- loon.ggplot(x, ...)
-  invisible(p)
+  print(p)
 }
 
-
+#' @title Reports whether x is a \code{l_ggplot} object
+#' @param x An object to test
+is.l_ggplot <- function(x) {
+  inherits(x, "is.l_ggplot")
+}
