@@ -1,36 +1,74 @@
-#' @title \code{loon} to \code{ggplot}
+#' @title Turn a \code{loon} widget to a \code{ggplot} object
 #'
 #' @description Create a \code{ggplot} object from a \code{loon} widget
 #'
-#' @param target a\code{loon} or a vector that specifies the
+#' @param target a \code{loon} or a vector that specifies the
 #' widget, layer, glyph, navigator or context completely.
 #' The widget is specified by the widget path name (e.g. '.l0.plot'),
 #' the remaining objects by their ids.
+#' @param asAes logical; set aesthetics attributes, i.e. `color`, `fill` as
+#' variables (default \code{TRUE}) or general visual properties (\code{FALSE}).
+#' See details
 #' @param ... arguments used inside \code{loon2ggplot()}, not used by this method
 #'
 #' @return a \code{ggplot} object
 #'
+#' @details
+#' in \code{ggplot}, generally speaking, there are two ways to set the
+#' aesthetics attributes, either take it as variables
+#' (set in function \code{aes()}) or visual properties. The main benefits to
+#' consider it as variables are that 1. legend could be shown; 2. convenient for
+#' further analysis. See examples.
+#'
 #' @export
 #' @examples
 #' if(interactive()) {
-#'   l <- l_plot(iris, color = iris$Species)
-#'   p <- loon2ggplot(l)
-#'   p # a ggplot object
-#'   str(p)
-#'   # add themes
-#'   p + geom_smooth() + theme_linedraw()
+#' lp <- l_plot(iris,
+#'              color = iris$Species,
+#'              glyph = "circle")
+#' gp <- loon2ggplot(lp)
+#' gp # a ggplot object
+#'
+#' # add smooth layer, grouped by color
+#' gp +
+#'   geom_smooth(aes(color = color)) +
+#'   # give meaningful legend label names
+#'   scale_color_manual(
+#'     # make sure the order is correct
+#'     values = unique(hex12tohex6(lp['color'])),
+#'     labels = c("setosa", "versicolor", "virginica")
+#'   )
+#'
+#' # histogram
+#' lh <- l_hist(mtcars$mpg,
+#'              color = factor(mtcars$gear))
+#'
+#' gh0 <- loon2ggplot(lh)
+#' # facet by `fill`
+#' gh0 + facet_wrap(~fill)
+#'
+#' # set `asAes = FALSE`
+#' gh1 <- loon2ggplot(lh, asAes = FALSE)
+#' # Expect the legend, they both are identical
+#' gh1
+#'
+#' \dontrun{
+#' # ERROR
+#' # The bins are constructed by `ggplot2::geom_rect()`
+#' # Very limited "fancy" operations can be made
+#' gh1 + facet_wrap(~fill)}
 #' }
 #'
-loon2ggplot <- function(target, ...) {
+loon2ggplot <- function(target, asAes = TRUE, ...) {
 
   if(ggplot2::is.ggplot(target) || is.ggmatrix(target)) {
     error_info <- deparse(substitute(target))
     stop(
-    "Target should be a loon object. Maybe you want to call `ggplot2loon(",
-    error_info,
-    ")`? Or, just call `loon.ggplot(",
-    error_info,
-    ")` for simplification.", call. = FALSE
+      "Target should be a loon object. Maybe you want to call `ggplot2loon(",
+      error_info,
+      ")`? Or, just call `loon.ggplot(",
+      error_info,
+      ")` for simplification.", call. = FALSE
     )
   }
   UseMethod('loon2ggplot', target)
@@ -38,7 +76,7 @@ loon2ggplot <- function(target, ...) {
 
 #' @export
 #' @rdname loon2ggplot
-loon2ggplot.default <- function(target, ...) {
+loon2ggplot.default <- function(target, asAes = TRUE, ...) {
   # TODO
   ggObj <- list(...)$ggObj
   ggObj
@@ -46,36 +84,47 @@ loon2ggplot.default <- function(target, ...) {
 
 #' @rdname loon2ggplot
 #' @export
-loon2ggplot.l_plot <- function(target, ...) {
+loon2ggplot.l_plot <- function(target, asAes = TRUE, ...) {
 
   loon::l_isLoonWidget(target) || stop("target does not exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
   cartesian_gg(target = target,
-               ggObj = loon2ggplot(rl))
+               ggObj = loon2ggplot(rl, asAes = asAes))
 }
 
 #' @rdname loon2ggplot
 #' @export
-loon2ggplot.l_hist <- function(target, ...) {
+loon2ggplot.l_hist <- function(target, asAes = TRUE, ...) {
 
   loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
+
+  if(target['yshows'] == "density" && length(unique(target['color'])) > 1) {
+    setLimits <- FALSE
+    message("In ggplot histogram, if `y` shows density, ",
+            "the area of each category (grouped by color) is 1; ",
+            "however, for an `l_hist` widget, ",
+            "the whole area is one and the area of each category is proportional to its counts.")
+  } else
+    setLimits <- TRUE
+
   cartesian_gg(target = target,
-               ggObj = loon2ggplot(rl))
+               ggObj = loon2ggplot(rl, asAes = asAes),
+               setLimits = setLimits)
 }
 
 #' @export
-loon2ggplot.l_graph <- function(target, ...) {
+loon2ggplot.l_graph <- function(target, asAes = TRUE, ...) {
 
   loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
   cartesian_gg(target = target,
-               ggObj = loon2ggplot(rl))
+               ggObj = loon2ggplot(rl, asAes = asAes))
 }
 
 #' @rdname loon2ggplot
 #' @export
-loon2ggplot.l_plot3D <- function(target, ...) {
+loon2ggplot.l_plot3D <- function(target,  asAes = TRUE, ...) {
 
   loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
@@ -101,7 +150,7 @@ loon2ggplot.l_plot3D <- function(target, ...) {
   y <- c(0.5, 0.5 + 0.08*axes_coords[[2]][1])
 
   cartesian_gg(target = target,
-               ggObj = loon2ggplot(rl)) +
+               ggObj = loon2ggplot(rl, asAes = asAes)) +
     ggplot2::geom_line(
       data = data.frame(
         x = x,
@@ -134,7 +183,7 @@ loon2ggplot.l_plot3D <- function(target, ...) {
     )
 }
 
-cartesian_gg <- function(target, ggObj) {
+cartesian_gg <- function(target, ggObj, setLimits = TRUE) {
 
   # keep name consistency
   widget <- target
@@ -183,7 +232,6 @@ cartesian_gg <- function(target, ggObj) {
   guidesBackGround <- if(showGuides) as_hex6color(widget['guidesBackground']) else as_hex6color(widget['background'])
 
   ggObj <- ggObj +
-    ggplot2::coord_cartesian(xlim = xlim, ylim = ylim) +
     ggplot2::xlab(xlabel) +
     ggplot2::ylab(ylabel) +
     ggplot2::ggtitle(label = if(title == "") NULL else title) +
@@ -209,47 +257,74 @@ cartesian_gg <- function(target, ggObj) {
                               size = 1) else ggplot2::element_blank(),
       plot.margin = grid::unit(margins, "lines")
     )
-  if(swapAxes) ggObj <- ggObj + coord_flip(xlim = xlim, ylim = ylim)
+
+  if(setLimits) {
+    ggObj <- ggObj +
+      ggplot2::coord_cartesian(xlim = xlim, ylim = ylim)
+
+    if(swapAxes) ggObj <- ggObj + coord_flip(xlim = xlim, ylim = ylim)
+
+  } else {
+    if(swapAxes) ggObj <- ggObj + coord_flip()
+  }
+
   return(ggObj)
 }
 
 #' @export
-loon2ggplot.l_layer_group <- function(target, ...) {
+loon2ggplot.l_layer_group <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   ggObj <- ggplot2::ggplot()
 
   children <- l_layer_getUngroupedChildren(widget = widget, target = widget)
-  l_children_layers <- lapply(rev(children),
-                              function(layerid) {
+  l_children_layers <- lapply(
+    rev(children),
+    function(layerid) {
 
-                                layer <- loon::l_create_handle(c(widget, layerid))
+      layer <- loon::l_create_handle(c(widget, layerid))
 
-                                if(layerid == 'model') {
+      if(layerid == 'model') {
 
-                                  states <- get_layer_states(widget, native_unit = FALSE)
+        states <- get_layer_states(widget, native_unit = FALSE)
 
-                                  if (length(states$x) == length(states$y)) {
-                                    # scatter plot
-                                    swapAxes <- widget["swapAxes"]
-                                    if(swapAxes) {
-                                      y <- states$x
-                                      x <- states$y
-                                    } else {
-                                      x <- states$x
-                                      y <- states$y
-                                    }
-                                    ggObj <<- ggplot2::ggplot(data = data.frame(x = x, y = y),
-                                                              mapping = ggplot2::aes(x = x, y = y))
-                                  } else if(length(states$y) == 1 && is.na(states$y)) {
-                                    # histogram
-                                    ggObj <<- ggplot2::ggplot(data = data.frame(x = states$x),
-                                                              mapping = ggplot2::aes(x = x))
-                                  } else NULL
-                                }
+        if(length(states$x) > 0) {
 
-                                layer
-                              })
+          if(inherits(widget, "l_hist")) {
+            # histogram
+            ggObj <<- ggplot2::ggplot(data = data.frame(x = states$x,
+                                                        color = hex2colorName(states$color),
+                                                        selected = states$selected,
+                                                        active = states$active),
+                                      mapping = ggplot2::aes(x = x))
+
+          } else {
+
+            # scatter plot
+            swapAxes <- widget["swapAxes"]
+            if(swapAxes) {
+              y <- states$x
+              x <- states$y
+            } else {
+              x <- states$x
+              y <- states$y
+            }
+
+            ggObj <<- ggplot2::ggplot(data = data.frame(x = x,
+                                                        y = y,
+                                                        glyph = states$glyph,
+                                                        size = states$size,
+                                                        color = hex2colorName(states$color),
+                                                        selected = states$selected,
+                                                        active = states$active),
+                                      mapping = ggplot2::aes(x = x,
+                                                             y = y))
+          }
+        } else NULL
+      }
+
+      layer
+    })
 
   l_visible_children_layer <- Filter(
     function(layerid) {
@@ -259,7 +334,7 @@ loon2ggplot.l_layer_group <- function(target, ...) {
   lapply(l_visible_children_layer,
          function(layer) {
 
-           ggObj <<- loon2ggplot(layer, ggObj = ggObj)
+           ggObj <<- loon2ggplot(layer, asAes = asAes, ggObj = ggObj)
          })
 
   ggObj
@@ -267,7 +342,7 @@ loon2ggplot.l_layer_group <- function(target, ...) {
 
 # primitive ggplot layers
 #' @export
-loon2ggplot.l_layer_polygon <- function(target, ...) {
+loon2ggplot.l_layer_polygon <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -288,8 +363,7 @@ loon2ggplot.l_layer_polygon <- function(target, ...) {
         mapping = ggplot2::aes(x = x, y = y),
         fill = states$color,
         colour = states$linecolor,
-        size =  as_r_line_size(states$linewidth),
-        inherit.aes = FALSE
+        size =  as_r_line_size(states$linewidth)
       )
   }
 
@@ -297,7 +371,7 @@ loon2ggplot.l_layer_polygon <- function(target, ...) {
 }
 
 #' @export
-loon2ggplot.l_layer_line <- function(target, ...) {
+loon2ggplot.l_layer_line <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -318,8 +392,7 @@ loon2ggplot.l_layer_line <- function(target, ...) {
         ),
         mapping = ggplot2::aes(x = x, y = y),
         colour = states$color,
-        size = as_r_line_size(states$linewidth),
-        inherit.aes = FALSE
+        size = as_r_line_size(states$linewidth)
       )
   }
 
@@ -327,7 +400,7 @@ loon2ggplot.l_layer_line <- function(target, ...) {
 }
 
 #' @export
-loon2ggplot.l_layer_rectangle <- function(target, ...) {
+loon2ggplot.l_layer_rectangle <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -346,19 +419,18 @@ loon2ggplot.l_layer_rectangle <- function(target, ...) {
           x = x,
           y = y
         ),
-        mapping = ggplot2::aes(xmin = x[1], xmax = x[2], ymin = y[1], ymax = y[2]),
+        mapping = ggplot2::aes(xmin = x[1], xmax = x[2],
+                               ymin = y[1], ymax = y[2]),
         colour = states$linecolor,
         fill = states$color,
-        size =  as_r_line_size(states$linewidth),
-        inherit.aes = FALSE
+        size =  as_r_line_size(states$linewidth)
       )
-
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_oval <- function(target, ...) {
+loon2ggplot.l_layer_oval <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -391,15 +463,14 @@ loon2ggplot.l_layer_oval <- function(target, ...) {
         mapping = ggplot2::aes(x = x, y = y),
         fill = states$color,
         colour = states$linecolor,
-        size = as_r_line_size(states$linewidth),
-        inherit.aes = FALSE
+        size = as_r_line_size(states$linewidth)
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_text <- function(target, ...) {
+loon2ggplot.l_layer_text <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -425,15 +496,14 @@ loon2ggplot.l_layer_text <- function(target, ...) {
         angle = states$angle,
         colour = states$color,
         hjust = get_hjust(states$justify),
-        size = as_r_text_size(states$size),
-        inherit.aes = FALSE
+        size = as_ggplot_size(states$size)
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_texts <- function(target, ...) {
+loon2ggplot.l_layer_texts <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -448,7 +518,7 @@ loon2ggplot.l_layer_texts <- function(target, ...) {
   if(length(x) > 0  & length(y) > 0){
 
     label  <- states$text[active]
-    size  <- as_r_text_size(states$size[active])
+    size  <- states$size[active]
     angle  <- states$angle[active]
     anchor  <- states$anchor[active]
     justify  <- states$justify[active]
@@ -479,15 +549,14 @@ loon2ggplot.l_layer_texts <- function(target, ...) {
         angle = angle,
         colour = color,
         hjust = get_hjust(justify),
-        size = as_r_text_size(size),
-        inherit.aes = FALSE
+        size = as_ggplot_size(size)
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_points <- function(target, ...) {
+loon2ggplot.l_layer_points <- function(target, asAes = TRUE, ...) {
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
   states <- get_layer_states(target, native_unit = FALSE)
@@ -499,24 +568,21 @@ loon2ggplot.l_layer_points <- function(target, ...) {
   ggObj <- list(...)$ggObj
 
   if(length(x) > 0  && length(y)  > 0) {
-    size  <- as_r_point_size(states$size[active])
-    color <- states$color[active]
 
     ggObj <- ggObj +
       ggplot2::geom_point(
         data = data.frame(x = x, y = y),
         mapping = ggplot2::aes(x = x, y = y),
-        colour = color,
-        size = size,
-        pch = 16,
-        inherit.aes = FALSE
+        colour =  states$color[active],
+        size = as_ggplot_size(states$size[active]),
+        pch = 16
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_polygons <- function(target, ...) {
+loon2ggplot.l_layer_polygons <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -549,15 +615,14 @@ loon2ggplot.l_layer_polygons <- function(target, ...) {
         mapping = ggplot2::aes(x = x, y = y, group = group),
         fill = rep(fill, times = len_x),
         colour = rep(linecolor, times = len_x),
-        size = rep(linewidth, times = len_x),
-        inherit.aes = FALSE
+        size = rep(linewidth, times = len_x)
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_rectangles <- function(target, ...) {
+loon2ggplot.l_layer_rectangles <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -586,18 +651,18 @@ loon2ggplot.l_layer_rectangles <- function(target, ...) {
     ggObj <- ggObj +
       ggplot2::geom_rect(
         data =df,
-        mapping = ggplot2::aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax),
+        mapping = ggplot2::aes(xmin = xmin, xmax = xmax,
+                               ymin = ymin, ymax = ymax),
         fill = states$color[active],
         colour = states$linecolor[active],
-        size = as_r_line_size(states$linewidth[active]),
-        inherit.aes = FALSE
+        size = as_r_line_size(states$linewidth[active])
       )
   }
 
   ggObj
 }
 #' @export
-loon2ggplot.l_layer_lines <- function(target, ...) {
+loon2ggplot.l_layer_lines <- function(target, asAes = TRUE, ...) {
 
   widget <- l_create_handle(attr(target, "widget"))
   swapAxes <- widget["swapAxes"]
@@ -616,7 +681,8 @@ loon2ggplot.l_layer_lines <- function(target, ...) {
 
     len_x <- lengths(x)
 
-    group <- as.factor(rep(1:length(len_x), times = len_x))
+    group <- as.factor(rep(seq(length(len_x)),
+                           times = len_x))
 
     df <- data.frame(
       x = as.numeric(unlist(x)),
@@ -629,8 +695,7 @@ loon2ggplot.l_layer_lines <- function(target, ...) {
         data =df,
         mapping = ggplot2::aes(x = x, y = y, group = group),
         colour = rep(linecolor, times = len_x),
-        size = rep(linewidth, times = len_x),
-        inherit.aes = FALSE
+        size = rep(linewidth, times = len_x)
       )
   }
 
