@@ -1,30 +1,24 @@
 #' @export
-loonGrob_layoutType.l_ggplot <- function(target) {
+loonGrob_layoutType.l_facet_ggplot <- function(target) {
   "arrangeGrobArgs"
 }
 
 #' @export
-l_get_arrangeGrobArgs.l_ggplot <- function(target){
-  plots <- target$plots
+l_get_arrangeGrobArgs.l_facet_ggplot <- function(target) {
+
+  plots <- l_getPlots(target)
   len_plots <- length(plots)
 
-  # label
-  xlabel <- paste0(unique(sapply(plots, function(plot)plot['xlabel'])), collapse = " ")
-  xlabel <- if (xlabel == "") NULL else xlabel
-  ylabel <- paste0(unique(sapply(plots, function(plot)plot['ylabel'])), collapse = " ")
-  ylabel <- if (ylabel == "") NULL else ylabel
-
-
-  # FacetWrap or FacetGrid
-  facet <- target$facet %||% list(FacetWrap = FALSE, FacetGrid = FALSE)
-  FacetWrap <- facet$FacetWrap
-  FacetGrid <- facet$FacetGrid
-
-  # title column subtitle or row subtitle
-  titles <- target$titles %||% list(title = "", colSubtitles = NULL, rowSubtitles = NULL)
-  title <- titles$title
-  colSubtitles <- titles$colSubtitles
-  rowSubtitles <- titles$rowSubtitles
+  subtitles <- l_getSubtitles(target)
+  xlabel <- subtitles$xlabel
+  ylabel <- subtitles$ylabel
+  title <- subtitles$title
+  FacetWrap <- subtitles$FacetWrap
+  FacetGrid <- subtitles$FacetGrid
+  colSubtitles <- subtitles$colSubtitles
+  rowSubtitles <- subtitles$rowSubtitles
+  byCOLS <- subtitles$byCOLS
+  byROWS <- subtitles$byROWS
 
   layout <- layout_coords(target)
   layoutDim <- apply(layout, 2, max)
@@ -36,10 +30,13 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
   # ttheme_minimal will give the same color background
   tt <- gridExtra::ttheme_minimal(
     base_size = 8,
-    core=list(bg_params = list(fill = "grey90", col=NA))
+    core=list(bg_params = list(fill = tryCatch(loon::l_getOption("facetLabelBackground"),
+                                               error = function(e) "gray80"),
+                               col = "white"))
   )
 
-  if(FacetWrap & length(colSubtitles) + length(rowSubtitles) > 0) {
+  if(FacetWrap && length(colSubtitles) + length(rowSubtitles) > 0) {
+
     subtitle <- c(colSubtitles, rowSubtitles)
     # loon grobs
     lgrobs <- do.call(gList,
@@ -52,7 +49,9 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
                                    grobs = setNames(
                                      lapply(rowi_columnIds,
                                             function(rowi_columnId){
-                                              loon::loonGrob(plots[[rowi_columnId]])
+                                              g <- loon::loonGrob(plots[[rowi_columnId]])
+                                              g$name <- as.character(plots[[rowi_columnId]])
+                                              g
                                             }
                                      ),
                                      as.character(plots[rowi_columnIds])
@@ -65,8 +64,7 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
 
                                  tG <- gridExtra::tableGrob(matrix(subtitle[rowi_columnIds],
                                                                    ncol = length(rowi_columnIds)),
-                                                            theme = tt
-                                 )
+                                                            theme = tt)
                                  rbind(tG, aGrob, size = "last")
                                } else grid::nullGrob(name = "null: no grob")
                              }
@@ -80,15 +78,17 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
       layout_matrix[nrow, ] <- c(rep(nrow, lastRowFacets) , rep(NA, (ncol - lastRowFacets)))
     }
 
-  } else if(FacetGrid & length(colSubtitles) + length(rowSubtitles) > 0) {
+  } else if(FacetGrid && length(colSubtitles) + length(rowSubtitles) > 0) {
 
-    if(facet$byCOLS & !facet$byROWS) {
+    if(byCOLS && !byROWS) {
       # loon grobs
       aGrob <- gridExtra::arrangeGrob(
         grobs = setNames(
           lapply(plots,
                  function(plot) {
-                   loon::loonGrob(plot)
+                   g <- loon::loonGrob(plot)
+                   g$name <- as.character(plot)
+                   g
                  }
           ),
           as.character(plots)
@@ -104,13 +104,15 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
         rbind(tG, aGrob, size = "last")
       )
 
-    } else if(!facet$byCOLS & facet$byROWS) {
+    } else if(!byCOLS && byROWS) {
       # loon grobs
       aGrob <- gridExtra::arrangeGrob(
         grobs = setNames(
           lapply(plots,
                  function(plot) {
-                   loon::loonGrob(plot)
+                   g <- loon::loonGrob(plot)
+                   g$name <- as.character(plot)
+                   g
                  }
           ),
           as.character(plots)
@@ -126,14 +128,17 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
         cbind(aGrob, tG, size = "first")
       )
 
-    } else if(facet$byCOLS & facet$byROWS) {
+    } else if(byCOLS && byROWS) {
+
       uniqueColSubtitles <- unique(colSubtitles)
       uniqueRowSubtitles <- unique(rowSubtitles)
       aGrob <- gridExtra::arrangeGrob(
         grobs = setNames(
           lapply(plots,
                  function(plot) {
-                   loon::loonGrob(plot)
+                   g <- loon::loonGrob(plot)
+                   g$name <- as.character(plot)
+                   g
                  }
           ), as.character(plots)
         ),
@@ -142,7 +147,8 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
         name = paste(c("byRow", "byColumn", "arrangeGrob"), collapse = " ")
       )
 
-      tG_row <- gridExtra::tableGrob(matrix(uniqueRowSubtitles, nrow = length(uniqueRowSubtitles)),
+      tG_row <- gridExtra::tableGrob(matrix(uniqueRowSubtitles,
+                                            nrow = length(uniqueRowSubtitles)),
                                      theme = tt)
       lgrobs_row <- cbind(aGrob, tG_row, size = "first")
 
@@ -161,6 +167,7 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
 
   } else {
 
+    ######### deprecated ###########
     # layout matrix
     layout_matrix <- matrix(rep(NA, nrow * ncol), nrow = nrow)
     for(i in 1:len_plots) {
@@ -178,7 +185,9 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
     lgrobs <- setNames(
       lapply(plots,
              function(plot) {
-               loon::loonGrob(plot)
+               g <- loon::loonGrob(plot)
+               g$name <- as.character(plot)
+               g
              }
       ), as.character(plots)
     )
@@ -190,23 +199,29 @@ l_get_arrangeGrobArgs.l_ggplot <- function(target){
     left = ylabel,
     bottom = xlabel,
     top = grid::textGrob(title, x = 0, hjust = 0),
-    name = "l_ggplot"
+    name = "l_facet_ggplot"
   )
 }
 
 
 #' @export
-l_getPlots.l_ggplot <- function(target){
-  plots  <- target$plots
+l_getPlots.l_facet_ggplot <- function(target) {
+  plots  <- unclass(target)
   lapply(plots,
-         function(plot){loon::l_throwErrorIfNotLoonWidget(plot) }
+         function(plot)
+           loon::l_throwErrorIfNotLoonWidget(plot)
   )
   plots
 }
 
 #' @export
-l_getLocations.l_ggplot <-  function(target){
-  plots <- target$plots
+l_getPlots.l_ggmatrix <- function(target) {
+  l_getPlots.l_facet_ggplot(target)
+}
+
+#' @export
+l_getLocations.l_facet_ggplot <-  function(target) {
+  plots <- l_getPlots(target)
   layout <- layout_coords(target)
   layoutDim <- apply(layout, 2, max)
   nrow <- layoutDim[1]
@@ -219,5 +234,117 @@ l_getLocations.l_ggplot <-  function(target){
     # set layout matrix
     layout_matrix[layout_i[1], layout_i[2]] <- i
   }
-  layout_matrix
+  list(
+    layout_matrix = layout_matrix,
+    nrow = nrow(layout_matrix),
+    ncol = ncol(layout_matrix)
+  )
+}
+
+#' @export
+l_getLocations.l_ggmatrix <- function(target) {
+  l_getLocations.l_facet_ggplot(target)
+}
+
+l_getSubtitles <- function(target) {
+
+  # find the parent `tk` window name
+  parent <- as.character(tkwinfo("parent",  target[[1L]]))
+  # access all children
+  children <- as.character(tkwinfo("child",  parent))
+
+  xLabelPathName <- children[grepl("xlabel", children)]
+  yLabelPathName <- children[grepl("ylabel", children)]
+  titlePathName <- children[grepl("title", children)]
+
+  # label
+  xlabel <- if(length(xLabelPathName) > 0) {
+    paste0(as.character(tkcget(xLabelPathName, "-text")), collapse = " ")
+  } else {
+    NULL
+  }
+  ylabel <- if(length(yLabelPathName) > 0) {
+    paste0(as.character(tkcget(yLabelPathName, "-text")), collapse = "")
+  } else {
+    NULL
+  }
+  title <- if(length(titlePathName) > 0) {
+    paste0(as.character(tkcget(titlePathName, "-text")), collapse = " ")
+  } else {
+    NULL
+  }
+
+  # FacetWrap or FacetGrid
+  FacetWrap <- any(grepl("wrap", children))
+  FacetGrid <- any(grepl("grid", children))
+
+  # title column subtitle or row subtitle
+  if(FacetWrap) {
+
+    columnlabelPathName <- children[grepl("columnlabel", children)]
+    colSubtitles <- if(length(columnlabelPathName) > 0L) {
+      unname(
+        sapply(columnlabelPathName,
+               function(path) {
+                 paste0(as.character(tkcget(path, "-text")), collapse = " ")
+               })
+      )
+    } else NULL
+    rowSubtitles <- NULL
+
+    labelPathName1L <- c(columnlabelPathName)[1L]
+    pathSplit <- strsplit(labelPathName1L, "-")[[1L]]
+    byCOLSChar <- pathSplit[grepl("byCOLS", pathSplit)]
+    byCOLS <- grepl("TRUE", byCOLSChar)
+    byROWSChar <- pathSplit[grepl("byROWS", pathSplit)]
+    byROWS <- grepl("TRUE", byROWSChar)
+
+  } else if (FacetGrid) {
+
+    columnlabelPathName <- children[grepl("columnlabel", children)]
+    rowlabelPathName <- children[grepl("rowlabel", children)]
+
+    colSubtitles <- if(length(columnlabelPathName) > 0) {
+      unname(
+        sapply(columnlabelPathName,
+               function(path) {
+                 paste0(as.character(tkcget(path, "-text")), collapse = " ")
+               })
+      )
+    } else NULL
+
+    rowSubtitles <- if(length(rowlabelPathName) > 0) {
+      unname(
+        sapply(rowlabelPathName,
+               function(path) {
+                 paste0(as.character(tkcget(path, "-text")), collapse = "")
+               })
+      )
+    } else NULL
+
+    labelPathName1L <- c(columnlabelPathName, rowlabelPathName)[1L]
+    pathSplit <- strsplit(labelPathName1L, "-")[[1L]]
+    byCOLSChar <- pathSplit[grepl("byCOLS", pathSplit)]
+    byCOLS <- grepl("TRUE", byCOLSChar)
+    byROWSChar <- pathSplit[grepl("byROWS", pathSplit)]
+    byROWS <- grepl("TRUE", byROWSChar)
+
+  } else {
+    colSubtitles <- NULL
+    rowSubtitles <- NULL
+    byCOLS <- FALSE
+    byROWS <- FALSE
+  }
+
+  list(
+    xlabel = xlabel,
+    ylabel = ylabel,
+    title = title,
+    FacetWrap = FacetWrap,
+    FacetGrid = FacetGrid,
+    colSubtitles = colSubtitles,
+    rowSubtitles = rowSubtitles,
+    byCOLS = byCOLS,
+    byROWS = byROWS
+  )
 }
